using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;

using BeeDevelopment.Brass3;

public class PalImage {

	private const int Width = 18;
	private const int Height = 304;

	public static void IncPalImage(Compiler compiler, string filename) {
	
		// Get values for the various colour levels:
		int[] ColourLevels = new int[] {
			(int)compiler.Labels["Colour.Sync"].NumericValue,
			(int)compiler.Labels["Colour.Black"].NumericValue,
			(int)compiler.Labels["Colour.Grey"].NumericValue,
			(int)compiler.Labels["Colour.White"].NumericValue
		};
		
		// Load the source image.
		using (Bitmap SourceImage = new Bitmap(compiler.ResolveFilename(filename))) {

			// Create a full-size bitmap to work with.
			using (Bitmap WorkingImage = new Bitmap(Width, Height, PixelFormat.Format32bppRgb)) {

				// Use a Graphics object to draw to the bitmap.
				using (Graphics WorkingGraphics = Graphics.FromImage(WorkingImage)) {
					WorkingGraphics.PixelOffsetMode = PixelOffsetMode.Half;
					WorkingGraphics.InterpolationMode = InterpolationMode.NearestNeighbor;

					// Scale the image to full size.
					WorkingGraphics.DrawImage(SourceImage, 0, 0, Width, Height);
				}

				// Dump the pixel data into an array.
				BitmapData GetImageData = WorkingImage.LockBits(new Rectangle(0, 0, Width, Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppRgb);
				int[] Pixels = new int[Height * GetImageData.Stride / 4];
				Marshal.Copy(GetImageData.Scan0, Pixels, 0, Pixels.Length);
				WorkingImage.UnlockBits(GetImageData);

				// Output the image data as bytes.
				for (int y = 0; y < Height; ++y) {
					for (int x = 0; x < Width; ++x) {
						int Pixel = Pixels[y * (GetImageData.Stride / 4) + x];
						int Brightness = ((Pixel & 0xFF) + ((Pixel >> 8) & 0xFF) + ((Pixel >> 16) & 0xFF)) / 256;
						compiler.WriteStaticOutput(
							(byte)ColourLevels[1 + Math.Min(2, Math.Max(0, Brightness))]
						);
					}
				}


			}

		}
	}
}